<?php

namespace Drupal\gatsby;

use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\ConnectException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\jsonapi_extras\EntityToJsonApi;

/**
 * Class GatsbyPreview.
 */
class GatsbyPreview {

  /**
   * GuzzleHttp\ClientInterface definition.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  private $httpClient;

  /**
   * Config Interface for accessing site configuration.
   *
   * @var Drupal\Core\Config\ConfigFactoryInterface
   */
  private $config;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * Drupal\Core\Logger\LoggerChannelFactory definition.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactory
   */
  private $logger;

  /**
   * Drupal\jsonapi_extras\EntityToJsonApi definition.
   *
   * @var \Drupal\jsonapi_extras\EntityToJsonApi
   */
  private $entityToJsonApi;

  /**
   * Constructs a new WebinarSync object.
   */
  public function __construct(ClientInterface $http_client,
      ConfigFactoryInterface $config,
      EntityTypeManagerInterface $entity_type_manager,
      LoggerChannelFactory $logger,
      EntityToJsonApi $entity_to_json_api) {
    $this->httpClient = $http_client;
    $this->config = $config->get('gatsby.settings');
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger->get('gatsby');
    $this->entityToJsonApi = $entity_to_json_api;
  }

  /**
   * Send updates to Gatsby Preview server.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to update.
   */
  public function updatePreviewEntity(ContentEntityInterface $entity) {
    $encoded_json = $this->entityToJsonApi->serialize($entity);

    $server_url = $this->config->get('server_url');

    try {
      $this->httpClient->post(
        $server_url . "/___updatePreview",
        [
          'json' => $encoded_json,
          'timeout' => 1,
        ]
      );
    }
    catch (ConnectException $e) {
      // Do nothing as no response is returned from the preview server.
    }
  }

  /**
   * Send delete request to Gatsby Preview server.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to delete.
   */
  public function deletePreviewEntity(ContentEntityInterface $entity) {
    $server_url = $this->config->get('server_url');

    try {
      $data = [
        'id' => $entity->uuid(),
        'action' => 'delete',
      ];

      $this->httpClient->post(
        $server_url . "/___updatePreview",
        [
          'json' => json_encode($data),
          'timeout' => 1,
        ]
      );
    }
    catch (ConnectException $e) {
      // Do nothing as no response is returned from the preview server.
    }
  }

  /**
   * Verify the entity is selected to sync to the Gatsby site.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   *
   * @return bool
   *   If the entity type should be sent to Gatsby Preview.
   */
  public function isPreviewEntity(ContentEntityInterface $entity) {
    $entityType = $entity->getEntityTypeId();
    $selectedEntityTypes = $this->config->get('preview_entity_types');
    return in_array($entityType, array_values($selectedEntityTypes), TRUE);
  }

}
